@@ -1,24 +1,25 @@
 import json
 import os
+from copy import deepcopy
 
 import numpy as np
 
 from .. import resolvers, util
 from ..base import Trimesh
 from ..exceptions import ExceptionWrapper
-from ..parent import Geometry
+from ..parent import Geometry, LoadSource
 from ..points import PointCloud
 from ..scene.scene import Scene, append_scenes
-from ..typed import Loadable, Optional
-from ..util import log, now
+from ..typed import Dict, Loadable, Optional, Set
+from ..util import log
 from . import misc
 from .binvox import _binvox_loaders
+from .cascade import _cascade_loaders
 from .dae import _collada_loaders
 from .gltf import _gltf_loaders
 from .misc import _misc_loaders
 from .obj import _obj_loaders
 from .off import _off_loaders
-from .openctm import _ctm_loaders
 from .ply import _ply_loaders
 from .stl import _stl_loaders
 from .threedxml import _threedxml_loaders
@@ -31,35 +32,36 @@ try:
 except BaseException as E:
     # save a traceback to see why path didn't import
     load_path = ExceptionWrapper(E)
-    # no path formats available
 
+    # no path formats available
     def path_formats() -> set:
         return set()
 
 
-def mesh_formats() -> set:
+def mesh_formats() -> Set[str]:
     """
     Get a list of mesh formats available to load.
 
     Returns
     -----------
-    loaders : list
-      Extensions of available mesh loaders,
-      i.e. 'stl', 'ply', etc.
+    loaders
+      Extensions of available mesh loaders
+      i.e. `{'stl', 'ply'}`
     """
     # filter out exceptionmodule loaders
     return {k for k, v in mesh_loaders.items() if not isinstance(v, ExceptionWrapper)}
 
 
-def available_formats() -> set:
+def available_formats() -> Set[str]:
     """
     Get a list of all available loaders
 
+
     Returns
     -----------
-    loaders : list
-        Extensions of available loaders
-        i.e. 'stl', 'ply', 'dxf', etc.
+    loaders
+      Extensions of all available loaders
+      i.e. `{'stl', 'ply', 'dxf'}`
     """
     loaders = mesh_formats()
     loaders.update(path_formats())
@@ -71,13 +73,17 @@ def available_formats() -> set:
 def load(
     file_obj: Loadable,
     file_type: Optional[str] = None,
-    resolver: Optional[resolvers.Resolver] = None,
+    resolver: Optional[resolvers.ResolverLike] = None,
     force: Optional[str] = None,
+    allow_remote: bool = False,
     **kwargs,
-):
+) -> Geometry:
     """
-    Load a mesh or vectorized path into objects like
-    Trimesh, Path2D, Path3D, Scene
+    THIS FUNCTION IS DEPRECATED but there are no current plans for it to be removed.
+
+    For new code the typed load functions `trimesh.load_scene` or `trimesh.load_mesh`
+    are recommended over `trimesh.load` which is a backwards-compatibility wrapper
+    that mimics the behavior of the old function and can return any geometry type.
 
     Parameters
     -----------
@@ -90,6 +96,8 @@ def load(
     force : None or str
       For 'mesh': try to coerce scenes into a single mesh
       For 'scene': try to coerce everything into a scene
+    allow_remote
+      If True allow this load call to work on a remote URL.
     kwargs : dict
       Passed to geometry __init__
 
@@ -98,77 +106,158 @@ def load(
     geometry : Trimesh, Path2D, Path3D, Scene
       Loaded geometry as trimesh classes
     """
-    # check to see if we're trying to load something
-    # that is already a native trimesh Geometry subclass
-    if isinstance(file_obj, Geometry):
-        log.info("Load called on %s object, returning input", file_obj.__class__.__name__)
-        return file_obj
 
-    # parse the file arguments into clean loadable form
-    (
-        file_obj,  # file- like object
-        file_type,  # str, what kind of file
-        metadata,  # dict, any metadata from file name
-        opened,  # bool, did we open the file ourselves
-        resolver,  # object to load referenced resources
-    ) = parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
+    # call the most general loading case into a `Scene`.
+    loaded = load_scene(
+        file_obj=file_obj,
+        file_type=file_type,
+        resolver=resolver,
+        allow_remote=allow_remote,
+        **kwargs,
+    )
 
-    try:
-        if isinstance(file_obj, dict):
-            # if we've been passed a dict treat it as kwargs
-            kwargs.update(file_obj)
-            loaded = load_kwargs(kwargs)
-        elif file_type in path_formats():
-            # path formats get loaded with path loader
-            loaded = load_path(file_obj, file_type=file_type, **kwargs)
-        elif file_type in mesh_loaders:
-            # mesh loaders use mesh loader
-            loaded = load_mesh(file_obj, file_type=file_type, resolver=resolver, **kwargs)
-        elif file_type in compressed_loaders:
-            # for archives, like ZIP files
-            loaded = load_compressed(file_obj, file_type=file_type, **kwargs)
-        elif file_type in voxel_loaders:
-            loaded = voxel_loaders[file_type](
-                file_obj, file_type=file_type, resolver=resolver, **kwargs
-            )
-        else:
-            if file_type in ["svg", "dxf"]:
-                # call the dummy function to raise the import error
-                # this prevents the exception from being super opaque
-                load_path()
-            else:
-                raise ValueError("File type: %s not supported" % file_type)
-    finally:
-        # close any opened files even if we crashed out
-        if opened:
-            file_obj.close()
+    if force == "mesh":
+        # new code should use `load_mesh` for this
+        log.debug(
+            "`trimesh.load(force='mesh')` is a compatibility wrapper for `trimesh.load_mesh`"
+        )
+        return loaded.to_mesh()
+    elif force == "scene":
+        # new code should use `load_scene` for this
+        log.debug(
+            "`trimesh.load(force='scene')` is a compatibility wrapper for `trimesh.load_scene`"
+        )
+        return loaded
 
-    # add load metadata ('file_name') to each loaded geometry
-    for i in util.make_sequence(loaded):
-        i.metadata.update(metadata)
+    ###########################################
+    # we are matching old, deprecated behavior here!
+    kind = loaded.source.file_type
+    always_scene = {"glb", "gltf", "zip", "3dxml", "tar.gz"}
 
-    # if we opened the file in this function ourselves from a
-    # file name clean up after ourselves by closing it
-    if opened:
-        file_obj.close()
+    if kind not in always_scene and len(loaded.geometry) == 1:
+        geom = next(iter(loaded.geometry.values()))
+        geom.metadata.update(loaded.metadata)
 
-    # combine a scene into a single mesh
-    if force == "mesh" and isinstance(loaded, Scene):
-        return util.concatenate(loaded.dump())
-    if force == "scene" and not isinstance(loaded, Scene):
-        return Scene(loaded)
+        if isinstance(geom, PointCloud) or kind in {
+            "obj",
+            "stl",
+            "ply",
+            "svg",
+            "binvox",
+            "xaml",
+            "dxf",
+            "off",
+            "msh",
+        }:
+            return geom
 
     return loaded
 
 
-def load_mesh(
+def load_scene(
     file_obj: Loadable,
     file_type: Optional[str] = None,
-    resolver: Optional[resolvers.Resolver] = None,
+    resolver: Optional[resolvers.ResolverLike] = None,
+    allow_remote: bool = False,
+    metadata: Optional[Dict] = None,
     **kwargs,
-):
+) -> Scene:
     """
-    Load a mesh file into a Trimesh object
+    Load geometry into the `trimesh.Scene` container. This may contain
+    any `parent.Geometry` object, including `Trimesh`, `Path2D`, `Path3D`,
+    or a `PointCloud`.
+
+    Parameters
+    -----------
+    file_obj : str, or file- like object
+      The source of the data to be loadeded
+    file_type: str
+      What kind of file type do we have (eg: 'stl')
+    resolver : trimesh.visual.Resolver
+      Object to load referenced assets like materials and textures
+    force : None or str
+      For 'mesh': try to coerce scenes into a single mesh
+      For 'scene': try to coerce everything into a scene
+    allow_remote
+      If True allow this load call to work on a remote URL.
+    kwargs : dict
+      Passed to geometry __init__
+
+    Returns
+    ---------
+    geometry : Trimesh, Path2D, Path3D, Scene
+      Loaded geometry as trimesh classes
+    """
+
+    # parse all possible values of file objects into simple types
+    arg = _parse_file_args(
+        file_obj=file_obj,
+        file_type=file_type,
+        resolver=resolver,
+        allow_remote=allow_remote,
+    )
+
+    try:
+        if isinstance(file_obj, dict):
+            # we've been passed a dictionary so treat them as keyword arguments
+            loaded = _load_kwargs(file_obj)
+        elif arg.file_type in path_formats():
+            # use path loader
+            loaded = load_path(
+                file_obj=arg.file_obj,
+                file_type=arg.file_type,
+                metadata=metadata,
+                **kwargs,
+            )
+        elif arg.file_type in mesh_loaders:
+            # use mesh loader
+            parsed = deepcopy(kwargs)
+            parsed.update(
+                mesh_loaders[arg.file_type](
+                    file_obj=arg.file_obj,
+                    file_type=arg.file_type,
+                    resolver=arg.resolver,
+                    metadata=metadata,
+                    **kwargs,
+                )
+            )
+            loaded = _load_kwargs(**parsed)
+
+        elif arg.file_type in compressed_loaders:
+            # for archives, like ZIP files
+            loaded = _load_compressed(arg.file_obj, file_type=arg.file_type, **kwargs)
+        elif arg.file_type in voxel_loaders:
+            loaded = voxel_loaders[arg.file_type](
+                file_obj=arg.file_obj,
+                file_type=arg.file_type,
+                resolver=arg.resolver,
+                **kwargs,
+            )
+        else:
+            raise NotImplementedError(f"file_type '{arg.file_type}' not supported")
+
+    finally:
+        # if we opened the file ourselves from a file name
+        # close any opened files even if we crashed out
+        if arg.was_opened:
+            arg.file_obj.close()
+
+    if not isinstance(loaded, Scene):
+        # file name may be used for nodes
+        loaded._source = arg
+        loaded = Scene(loaded)
+
+    # add on the loading information
+    loaded._source = arg
+    for g in loaded.geometry.values():
+        g._source = arg
+
+    return loaded
+
+
+def load_mesh(*args, **kwargs) -> Trimesh:
+    """
+    Load a file into a Trimesh object.
 
     Parameters
     -----------
@@ -181,48 +270,13 @@ def load_mesh(
 
     Returns
     ----------
-    mesh : trimesh.Trimesh or trimesh.Scene
-      Loaded geometry data
+    mesh
+      Loaded geometry data.
     """
-
-    # parse the file arguments into clean loadable form
-    (
-        file_obj,  # file- like object
-        file_type,  # str, what kind of file
-        metadata,  # dict, any metadata from file name
-        opened,  # bool, did we open the file ourselves
-        resolver,  # object to load referenced resources
-    ) = parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
-
-    try:
-        # make sure we keep passed kwargs to loader
-        # but also make sure loader keys override passed keys
-        loader = mesh_loaders[file_type]
-        tic = now()
-        results = loader(file_obj, file_type=file_type, resolver=resolver, **kwargs)
-        if not isinstance(results, list):
-            results = [results]
-
-        loaded = []
-        for result in results:
-            kwargs.update(result)
-            loaded.append(load_kwargs(kwargs))
-            loaded[-1].metadata.update(metadata)
-        if len(loaded) == 1:
-            loaded = loaded[0]
-        # show the repr for loaded, loader used, and time
-        log.debug(
-            f"loaded {str(loaded)} using `{loader.__name__}` in {now() - tic:0.4f}s"
-        )
-    finally:
-        # if we failed to load close file
-        if opened:
-            file_obj.close()
-
-    return loaded
+    return load_scene(*args, **kwargs).to_mesh()
 
 
-def load_compressed(file_obj, file_type=None, resolver=None, mixed=False, **kwargs):
+def _load_compressed(file_obj, file_type=None, resolver=None, mixed=False, **kwargs):
     """
     Given a compressed archive load all the geometry that
     we can from it.
@@ -244,86 +298,68 @@ def load_compressed(file_obj, file_type=None, resolver=None, mixed=False, **kwar
     """
 
     # parse the file arguments into clean loadable form
-    (
-        file_obj,  # file- like object
-        file_type,  # str, what kind of file
-        metadata,  # dict, any metadata from file name
-        opened,  # bool, did we open the file ourselves
-        resolver,  # object to load referenced resources
-    ) = parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
+    arg = _parse_file_args(file_obj=file_obj, file_type=file_type, resolver=resolver)
 
-    try:
-        # a dict of 'name' : file-like object
-        files = util.decompress(file_obj=file_obj, file_type=file_type)
-        # store loaded geometries as a list
-        geometries = []
+    # store loaded geometries as a list
+    geometries = []
 
-        # so loaders can access textures/etc
-        resolver = resolvers.ZipResolver(files)
+    # so loaders can access textures/etc
+    archive = util.decompress(file_obj=arg.file_obj, file_type=arg.file_type)
+    resolver = resolvers.ZipResolver(archive)
 
-        # try to save the files with meaningful metadata
-        if "file_path" in metadata:
-            archive_name = metadata["file_path"]
+    # try to save the files with meaningful metadata
+    # archive_name = arg.file_path or "archive"
+    meta_archive = {}
+
+    # populate our available formats
+    if mixed:
+        available = available_formats()
+    else:
+        # all types contained in ZIP archive
+        contains = {util.split_extension(n).lower() for n in resolver.keys()}
+        # if there are no mesh formats available
+        if contains.isdisjoint(mesh_formats()):
+            available = path_formats()
         else:
-            archive_name = "archive"
+            available = mesh_formats()
 
-        # populate our available formats
-        if mixed:
-            available = available_formats()
-        else:
-            # all types contained in ZIP archive
-            contains = {util.split_extension(n).lower() for n in files.keys()}
-            # if there are no mesh formats available
-            if contains.isdisjoint(mesh_formats()):
-                available = path_formats()
-            else:
-                available = mesh_formats()
+    for file_name, file_obj in archive.items():
+        try:
+            # only load formats that we support
+            compressed_type = util.split_extension(file_name).lower()
 
-        meta_archive = {}
-        for name, data in files.items():
-            try:
-                # only load formats that we support
-                compressed_type = util.split_extension(name).lower()
+            # if file has metadata type include it
+            if compressed_type in ("yaml", "yml"):
+                import yaml
 
-                # if file has metadata type include it
-                if compressed_type in "yaml":
-                    import yaml
+                continue
+                meta_archive[file_name] = yaml.safe_load(file_obj)
+            elif compressed_type == "json":
+                import json
 
-                    meta_archive[name] = yaml.safe_load(data)
-                elif compressed_type in "json":
-                    import json
+                meta_archive[file_name] = json.load(file_obj)
+                continue
+            elif compressed_type not in available:
+                # don't raise an exception, just try the next one
+                continue
 
-                    meta_archive[name] = json.loads(data)
-
-                if compressed_type not in available:
-                    # don't raise an exception, just try the next one
-                    continue
-                # store the file name relative to the archive
-                metadata["file_name"] = archive_name + "/" + os.path.basename(name)
-                # load the individual geometry
-                loaded = load(
-                    file_obj=data,
+            # load the individual geometry
+            geometries.append(
+                load_scene(
+                    file_obj=file_obj,
                     file_type=compressed_type,
                     resolver=resolver,
-                    metadata=metadata,
                     **kwargs,
                 )
+            )
 
-                # some loaders return multiple geometries
-                if util.is_sequence(loaded):
-                    # if the loader has returned a list of meshes
-                    geometries.extend(loaded)
-                else:
-                    # if the loader has returned a single geometry
-                    geometries.append(loaded)
-            except BaseException:
-                log.debug("failed to load file in zip", exc_info=True)
+        except BaseException:
+            log.debug("failed to load file in zip", exc_info=True)
 
-    finally:
-        # if we opened the file in this function
-        # clean up after ourselves
-        if opened:
-            file_obj.close()
+    # if we opened the file in this function
+    # clean up after ourselves
+    if arg.was_opened:
+        arg.file_obj.close()
 
     # append meshes or scenes into a single Scene object
     result = append_scenes(geometries)
@@ -335,61 +371,34 @@ def load_compressed(file_obj, file_type=None, resolver=None, mixed=False, **kwar
     return result
 
 
-def load_remote(url, **kwargs):
+def load_remote(url: str, **kwargs) -> Scene:
     """
     Load a mesh at a remote URL into a local trimesh object.
 
-    This must be called explicitly rather than automatically
-    from trimesh.load to ensure users don't accidentally make
-    network requests.
+    This is a thin wrapper around:
+      `trimesh.load_scene(file_obj=url, allow_remote=True, **kwargs)`
 
     Parameters
     ------------
-    url : string
+    url
       URL containing mesh file
-    **kwargs : passed to `load`
+    **kwargs
+      Passed to `load_scene`
 
     Returns
     ------------
     loaded : Trimesh, Path, Scene
       Loaded result
     """
-    # import here to keep requirement soft
-    import httpx
-
-    # download the mesh
-    response = httpx.get(url, follow_redirects=True)
-    response.raise_for_status()
-
-    # wrap as file object
-    file_obj = util.wrap_as_stream(response.content)
-
-    # so loaders can access textures/etc
-    resolver = resolvers.WebResolver(url)
-
-    try:
-        # if we have a bunch of query parameters the type
-        # will be wrong so try to clean up the URL
-        # urllib is Python 3 only
-        import urllib
-
-        # remove the url-safe encoding then split off query params
-        file_type = urllib.parse.unquote(url).split("?", 1)[0].split("/")[-1].strip()
-    except BaseException:
-        # otherwise just use the last chunk of URL
-        file_type = url.split("/")[-1].split("?", 1)[0]
-
-    # actually load the data from the retrieved bytes
-    loaded = load(file_obj=file_obj, file_type=file_type, resolver=resolver, **kwargs)
-    return loaded
+    return load_scene(file_obj=url, allow_remote=True, **kwargs)
 
 
-def load_kwargs(*args, **kwargs):
+def _load_kwargs(*args, **kwargs) -> Geometry:
     """
     Load geometry from a properly formatted dict or kwargs
     """
 
-    def handle_scene():
+    def handle_scene() -> Scene:
         """
         Load a scene from our kwargs.
 
@@ -399,7 +408,7 @@ def load_kwargs(*args, **kwargs):
         base_frame: str, base frame of graph
         """
         graph = kwargs.get("graph", None)
-        geometry = {k: load_kwargs(v) for k, v in kwargs["geometry"].items()}
+        geometry = {k: _load_kwargs(v) for k, v in kwargs["geometry"].items()}
 
         if graph is not None:
             scene = Scene()
@@ -412,6 +421,12 @@ def load_kwargs(*args, **kwargs):
         else:
             scene = Scene(geometry)
 
+        # camera, if it exists
+        camera = kwargs.get("camera")
+        if camera:
+            scene.camera = camera
+            scene.camera_transform = kwargs.get("camera_transform")
+
         if "base_frame" in kwargs:
             scene.graph.base_frame = kwargs["base_frame"]
         metadata = kwargs.get("metadata")
@@ -436,7 +451,7 @@ def load_kwargs(*args, **kwargs):
 
         return scene
 
-    def handle_mesh():
+    def handle_mesh() -> Trimesh:
         """
         Handle the keyword arguments for a Trimesh object
         """
@@ -451,10 +466,12 @@ def load_kwargs(*args, **kwargs):
         Handle an exported mesh.
         """
         data, file_type = kwargs["data"], kwargs["file_type"]
-        if not isinstance(data, dict):
-            data = util.wrap_as_stream(data)
-        k = mesh_loaders[file_type](data, file_type=file_type)
-        return Trimesh(**k)
+        if isinstance(data, dict):
+            return _load_kwargs(data)
+        elif file_type in mesh_loaders:
+            return Trimesh(**mesh_loaders[file_type](data, file_type=file_type))
+
+        raise NotImplementedError(f"`{file_type}` is not supported")
 
     def handle_path():
         from ..path import Path2D, Path3D
@@ -493,21 +510,18 @@ def load_kwargs(*args, **kwargs):
     for func, expected in handlers:
         if all(i in kwargs for i in expected):
             # all expected kwargs exist
-            handler = func
-            # exit the loop as we found one
-            break
-    else:
-        raise ValueError(f"unable to determine type: {kwargs.keys()}")
+            return func()
 
-    return handler()
+    raise ValueError(f"unable to determine type: {kwargs.keys()}")
 
 
-def parse_file_args(
-    file_obj: Loadable,
+def _parse_file_args(
+    file_obj,
     file_type: Optional[str],
-    resolver: Optional[resolvers.Resolver] = None,
+    resolver: Optional[resolvers.ResolverLike] = None,
+    allow_remote: bool = False,
     **kwargs,
-):
+) -> LoadSource:
     """
     Given a file_obj and a file_type try to magically convert
     arguments to a file-like object and a lowercase string of
@@ -546,37 +560,32 @@ def parse_file_args(
 
     Returns
     -----------
-    file_obj : file-like object
-      Contains data
-    file_type : str
-      Lower case of the type of file (eg 'stl', 'dae', etc)
-    metadata : dict
-      Any metadata gathered
-    opened : bool
-      Did we open the file or not
-    resolver : trimesh.visual.Resolver
-      Resolver to load other assets
+    args
+      Populated `_FileArg` message
     """
-    metadata = {}
-    opened = False
-    if "metadata" in kwargs and isinstance(kwargs["metadata"], dict):
-        metadata.update(kwargs["metadata"])
+    # try to save a file path from various inputs
+    file_path = None
+
+    # keep track if we opened a file ourselves and thus are
+    # responsible for closing it at the end of loading
+    was_opened = False
 
     if util.is_pathlib(file_obj):
         # convert pathlib objects to string
         file_obj = str(file_obj.absolute())
 
     if util.is_file(file_obj) and file_type is None:
-        raise ValueError("file_type must be set for file objects!")
-    if util.is_string(file_obj):
+        raise ValueError("`file_type` must be set for file objects!")
+
+    if isinstance(file_obj, str):
         try:
-            # os.path.isfile will return False incorrectly
-            # if we don't give it an absolute path
-            file_path = os.path.expanduser(file_obj)
-            file_path = os.path.abspath(file_path)
+            # clean up file path to an absolute location
+            file_path = os.path.abspath(os.path.expanduser(file_obj))
+            # check to see if this path exists
             exists = os.path.isfile(file_path)
         except BaseException:
             exists = False
+            file_path = None
 
         # file obj is a string which exists on filesystm
         if exists:
@@ -584,41 +593,49 @@ def parse_file_args(
             if resolver is None:
                 resolver = resolvers.FilePathResolver(file_path)
             # save the file name and path to metadata
-            metadata["file_path"] = file_path
-            metadata["file_name"] = os.path.basename(file_obj)
             # if file_obj is a path that exists use extension as file_type
             if file_type is None:
                 file_type = util.split_extension(file_path, special=["tar.gz", "tar.bz2"])
             # actually open the file
             file_obj = open(file_path, "rb")
-            opened = True
+            # save that we opened it so we can cleanup later
+            was_opened = True
         else:
             if "{" in file_obj:
-                # if a dict bracket is in the string, its probably a straight
-                # JSON
+                # if a bracket is in the string it's probably straight JSON
                 file_type = "json"
+                file_obj = util.wrap_as_stream(file_obj)
             elif "https://" in file_obj or "http://" in file_obj:
-                # we've been passed a URL, warn to use explicit function
-                # and don't do network calls via magical pipeline
-                raise ValueError(f"use load_remote to load URL: {file_obj}")
+                if not allow_remote:
+                    raise ValueError("unable to load URL with `allow_remote=False`")
+
+                import urllib
+
+                # remove the url-safe encoding and query params
+                file_type = util.split_extension(
+                    urllib.parse.unquote(file_obj).split("?", 1)[0].split("/")[-1].strip()
+                )
+                # create a web resolver to do the fetching and whatnot
+                resolver = resolvers.WebResolver(url=file_obj)
+                # fetch the base file
+                file_obj = util.wrap_as_stream(resolver.get_base())
+
             elif file_type is None:
-                raise ValueError(f"string is not a file: {file_obj}")
+                raise ValueError(f"string is not a file: `{file_obj}`")
 
-    if file_type is None:
-        file_type = file_obj.__class__.__name__
-
-    if util.is_string(file_type) and "." in file_type:
+    if isinstance(file_type, str) and "." in file_type:
         # if someone has passed the whole filename as the file_type
         # use the file extension as the file_type
-        if "file_path" not in metadata:
-            metadata["file_path"] = file_type
-        metadata["file_name"] = os.path.basename(file_type)
+        path = os.path.abspath(os.path.expanduser(file_type))
         file_type = util.split_extension(file_type)
-        if resolver is None and os.path.exists(file_type):
-            resolver = resolvers.FilePathResolver(file_type)
+        if os.path.exists(path):
+            file_path = path
+            if resolver is None:
+                resolver = resolvers.FilePathResolver(file_path)
 
     # all our stored extensions reference in lower case
-    file_type = file_type.lower()
+    if file_type is not None:
+        file_type = file_type.lower()
 
     # if we still have no resolver try using file_obj name
     if (
@@ -629,22 +646,27 @@ def parse_file_args(
     ):
         resolver = resolvers.FilePathResolver(file_obj.name)
 
-    return file_obj, file_type, metadata, opened, resolver
+    return LoadSource(
+        file_obj=file_obj,
+        file_type=file_type,
+        file_path=file_path,
+        was_opened=was_opened,
+        resolver=resolver,
+    )
 
 
 # loader functions for compressed extensions
 compressed_loaders = {
-    "zip": load_compressed,
-    "tar.bz2": load_compressed,
-    "tar.gz": load_compressed,
-    "bz2": load_compressed,
+    "zip": _load_compressed,
+    "tar.bz2": _load_compressed,
+    "tar.gz": _load_compressed,
+    "bz2": _load_compressed,
 }
 
 # map file_type to loader function
 mesh_loaders = {}
 mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_stl_loaders)
-mesh_loaders.update(_ctm_loaders)
 mesh_loaders.update(_ply_loaders)
 mesh_loaders.update(_obj_loaders)
 mesh_loaders.update(_off_loaders)
@@ -654,6 +676,7 @@ mesh_loaders.update(_xaml_loaders)
 mesh_loaders.update(_threedxml_loaders)
 mesh_loaders.update(_three_loaders)
 mesh_loaders.update(_xyz_loaders)
+mesh_loaders.update(_cascade_loaders)
 
 # collect loaders which return voxel types
 voxel_loaders = {}
